#include "main.h"

#define MIT_NAMEN	1		// 0= ohne, 1=mit Namen in DS1820

void start_meas( void ){
  if( W1_IN & 1<< W1_PIN ){
    w1_command( CONVERT_T, NULL );
    W1_OUT |= 1<< W1_PIN;
    W1_DDR |= 1<< W1_PIN;			// parasite power on

  }else{
  set_cursor(0,2);
    lcd_string( "Short Circuit !" );
  }
}

#if MIT_NAMEN
#define	DS18B20_12bit	0x7F
#define THERMOanzahlMax	3	// auch thermoNamen anpassen!

//                      "123456123456123456"
#define THERMO1WNAMEN	"    T1    T2    T3"	// berschrift, rechtsbndig
//                      " 23.0C 40.3C  2.5C"
#define THERMO1NameLen	6
#define THERMO1NameOffset	(THERMO1NameLen-2) // hier steht das 1. Zeichen
static	char	thermoNamen[] =	THERMO1WNAMEN;	// ToDo PROGMEM
static	int16_t	thermoValues[THERMOanzahlMax];
static	uint8_t initReady;
#if (THERMOanzahlMax<=8)
	static uint8_t valid; 	// Indizes mit gltigen Namen merken, je 1 bit
#else
	static uint16_t valid;
#endif

static uint8_t NameValid( char * name ) {
	// prft, ob Name aus scratch mit Name aus Tabelle bereinstimmt
	// wenn ja, ret = index+1!!!; andernfalls 0
	uint8_t ret = 0;
	char * pC = thermoNamen+THERMO1NameOffset;
	while (ret++ < THERMOanzahlMax) {
		if ( *(int16_t*) name== *(int16_t*) pC)	// 2 Byte vergleichen
			return ret;
		pC += THERMO1NameLen;
	}
	return 0;
}
#endif

void read_meas( void )
{
  uchar id[8], diff;
  uchar s[30];
  uchar i;
  uint temp;

  for( diff = SEARCH_FIRST; diff != LAST_DEVICE; ){
    diff = w1_rom_search( diff, id );

    if( diff == PRESENCE_ERR ){
	set_cursor(0,2);
    lcd_string( "No Sensor found" );
      break;
    }
    if( diff == DATA_ERR ){
	set_cursor(0,2);
      lcd_string( "Bus Error" );
      break;
    }
    if( id[0] == 0x28 || id[0] == 0x10 ){	// temperature sensor
#if !MIT_NAMEN
		set_cursor(0,1);
		lcd_string( "ID:" );

		//Wert der ID ausgeben
		for( i = 0; i < 8; i++ ){
			sprintf( (char*)s, "%02X ", id[i] );
			lcd_string( (char*)s );
	}
#endif
      w1_byte_wr( READ );			// read command
      temp = w1_byte_rd();			// low byte
      temp |= (uint)w1_byte_rd() << 8;		// high byte
      if( id[0] == 0x10 )			// 9 -> 12 bit
        temp <<= 3;

	  /* //Messwert als Hex auf LCD ausgeben
      sprintf( (char*)s, "%04Xh", temp );	// hex value
	  set_cursor(9,2);
      lcd_string( (char*)s );
      */
#if MIT_NAMEN
		char name[2]; uint8_t nIndex;
		name[0] = w1_byte_rd();			// high Index
		name[1] = w1_byte_rd();			// low	Index
		if (!initReady)	{			// nur beim 1. Durchgang
			// PrintId( id, name );	//	erkannte Sensoren mit Namen ausgeben, z.B. auf Uart
			set_cursor(0,1);		// berschrift in 1. Zeile LCD
			lcd_string( THERMO1WNAMEN );
		}
		if ((nIndex = NameValid(name))) {
			thermoValues[--nIndex] = temp;
			valid |= (1<<nIndex);
		} else if (initReady==1){ // alle im 0. Durchgang als gltig erkannten sind ausnahmsweise noch gesetzt
			i = 0;
			while ((1<<i++) & valid); 	// 1. freien Platz finden (i ist dann 1 zu gro)
			if (--i<THERMOanzahlMax) {	// also vorher 1 abziehen
				valid |= (1<<i);					// jetzt belegt
				i *= THERMO1NameLen;
				i += THERMO1NameOffset;
				w1_command( WRITE, id );
				w1_byte_wr( name[0]=thermoNamen[i++] );
				w1_byte_wr( name[1]=thermoNamen[i] );
				w1_byte_wr( DS18B20_12bit );	// ggf. anpassen fr anderen Sensor!
				w1_command( EE_WRITE, id );
				// PrintId( id, name );			// Namensnderung protokollieren
			}
		}
#else
	  //Messwert in Grad Celsius auf LCD ausgeben
      sprintf( (char*)s, "%3d.%01dC", temp >> 4, (temp << 12) / 6553 ); // 0.1C
		  // Achtung: liefert bei negativen Temperaturen falsche Ergebnisse!
	  set_cursor(2,2);
      lcd_string( (char*)s );
#endif
    }
  }
  //uputsnl( "" );
#if MIT_NAMEN
  set_cursor(0,2);	// Die entsprecheden Stellen in main.c mssen dann natrlich entfernt werden!
	for( i=0; i<THERMOanzahlMax; i++) {	// Error-Detection
		/* es treten gelegentlich Lesefehler auf (Detektor wird nicht gefunden).
		 * 	Ausgelassene Werte machen viel Arbeit in Excel und verunstalten die Grafik.
		 * 	Deshalb jetzt: alten Messwert +/- Konstante, z.B. 1=0x10 (1*16).
		 * 	und das letzte Bit gibt an, ob addiert oder subtrahiert werden muss (abwechselnd)
		 *  Wenn man das nicht braucht, kann man auch einfach einen sonst nicht vorkommenden
		 *  Wert einsetzen, z.B. 0.
		*/
		#define DEFoffset		0x10			// = 1
		#define DEFmask			((int16_t) 0x1)
		int16_t	temp=thermoValues[i];
		if (!(valid &(1<<i))) {	// keinen neuen Wert gelesen ?
			// dann den alten nehmen und eine wechselnde Markierung anbringen
			if ( temp & DEFmask)	// bit gesetzt?
					// dann subtrahieren und bit lschen
				temp = (temp-DEFoffset) & ~DEFmask;
			else 	// addieren und bit setzen
				temp = (temp+DEFoffset) | DEFmask;
			thermoValues[i]=temp;
		}
		sprintf( (char*)s, "%3d.%01dC", temp >> 4, (temp << 12) / 6553 ); // 0.1C
		/* die berschrift muss ggf. an die Stellenzahl dieser Ausgabe angepasst werden
		 * Achtung: liefert bei negativen Temperaturen falsche Ergebnisse!
		 * bei zu wenig Platz kann man auch den Dezimalpunkt weglassen und Dezigrad anzeigen
		 * sprintf( (char*)s, "%4d", (temp*10)>>4 ); // . und C weggelassen
		 * dann erspart man sich auch die Probleme bei negativen Temperaturen
		 * man kann den Dezimalpunkt auch nachtglich selbst einfgen:
		 * sprintf( (char*)s, "%4d_", (temp*10)>>4 ); // '_' wird in nchster Zeile berschrieben
		 * s[5]=s[4]; s[4]='.'; // man muss dann ggf. aber z.B. mit ".1" oder "-.5" leben.
		 */
		lcd_string( (char*)s );
	}
	if (initReady)
		valid = 0;
		// d.h. beim ersten Durchgang stehen lassen fr Erkennung der freien Pltze fr unbekannte Sensoren
	if (initReady<255)	// berlauf verhindern
		++initReady;
#endif
}
